;*******************************************************************************
;*
;*	Titel:	Unterprogrammbibliothek mit VDrive2 Befehlen (SR_VDrive2.asm)
;*	Datum:	08/03/08	Version:	0.1
;*	Autor:	WEIGU
;*
;* Informationen zur Beschaltung:
;*
;*      Eingaenge:      TxD vom VDrive an RxD (PD0)
;*                      (CTS vom VDrive auf Masse (GND))
;*      Ausgaenge:      RxD vom VDrive an TxD (PD1)
;*
;*
;*	Informationen zur Funktionsweise:
;*      Die Ansteuerung erfolgt ueber die serielle Schnittstelle ohne
;*      Hardware-Handshake (CTS auf Masse).
;*      Mit 1 MBit/s traten Probleme beim Schreiben von mehr als 60 Byte in
;*      eine Datei auf. Mit 115200 Bit/s traten keine Probleme auf.
;*
;*      Folgende Adressen im SRAM werden verwendet (Deklaration im Hauptprogr.):
;*      .DSEG                          ;was ab hier folgt kommt in den SRAM
;*
;*      ERR:    .BYTE   1              ;ERRorbyte
;*      BRDIV:  .BYTE   3              ;DIVisor fuer BaudRate (1. Byte in BRDIV)
;*      FWVER:  .BYTE   12             ;FirmWare VERsion (MAIN )dd(.)ddAAAAA
;*                                     ;                 (RPRG )d(.)dd(R)
;*      WRFNB:  .BYTE   1              ;Anzahl der Bytes fuer Schreibbefehl WRF
;*                                     ;(max 256 = 0xFF)
;*      RDFNB:  .BYTE   1              ;Anzahl der Bytes fuer Lesebefehl WRF
;*                                     ;(max 256 = 0xFF)
;*      FNAME:  .BYTE   13             ;Filename Dateiname max. 8+1(dot)+3+1(cr)
;*      FSIZE:  .BYTE   4              ;Filesize Dateigroesse 4 Bytes
;*      .ORG    0x0200
;*      DIRLST: .BYTE   256            ;Nach einem DIR Kommando wird in DIRLST
;*                                     ;die Anzahl der Dateien gespeichert
;*                                     ;Die zurueckgegebenen ASCII-Zeichen
;*                                     ;werden ab DIRLST+1 abgelegt.
;*      WDATA:  .BYTE   256            ;Daten zum Schreiben in eine Datei
;*      RDATA:  .BYTE   256            ;Daten zum Lesen aus einer Datei
;*
;*      Achtung: Zuerst auf Kurzbefehlsmodus umschalten (SCS) da alle Befehle
;*      fuer diesen Modus programmiert wurden.
;*      Weitere Infos im Firmware User Manual des VDRIVE2
;*
;*******************************************************************************

;-------------------------------------------------------------------------------
; SCS   Switches to the Shortened Command Set
;-------------------------------------------------------------------------------
SCS:    ldi     r24,0x10        ;send 0x10 0x0D
        rcall   SNDCHR
        ldi     r24,0x0D
        rcall   SNDCHR
        rcall   RCVCHR          ;test monitor prompt ">" (0x3e 0x0d)
        cpi     r24,0x3E        ;if no prompt, set ERRorbyte (ERR)
        brne    SCSE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    SCSE
        ret                     ;back
SCSE:   lds     r24,ERR
        ori     r24,0x01        ;set Bit 0 in ERRorbyte
        sts     ERR,r24
        ret                     ;back

;-------------------------------------------------------------------------------
; ECS   Switches to Extended Command Set
;-------------------------------------------------------------------------------
ECS:    ldi     r24,0x11       ;send 0x11 0x0D
        rcall   SNDCHR
        ldi     r24,0x0D
        rcall   SNDCHR
        rcall   RCVCHR          ;test monitor prompt "D:\>"
        cpi     r24,0x44       ;(0x44 0x3a 0x5c 0x3e 0x0d)
        brne    ECSE            ;if no prompt, set ERRorbyte (ERR)
        rcall   RCVCHR
        cpi     r24,0x3A
        brne    ECSE
        rcall   RCVCHR
        cpi     r24,0x5C
        brne    ECSE
        rcall   RCVCHR
        cpi     r24,0x3E
        brne    ECSE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    ECSE
        ret                     ;back
ECSE:   lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte (0x200 in SRAM)
        sts     ERR,r24
        ret                     ;back

;-------------------------------------------------------------------------------
; IPA   Monitor commands use ASCII values (shortened command!)
;-------------------------------------------------------------------------------
IPA:    ldi     r24,0x90       ;send 0x90 0x0D
        rcall   SNDCHR
        ldi     r24,0x0D
        rcall   SNDCHR
        rcall   RCVCHR          ;test monitor prompt ">" (0x3e 0x0d)
        cpi     r24,0x3E       ;if no prompt, set ERRorbyte (ERR)
        brne    IPAE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    IPAE
        ret                     ;back
IPAE:   lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte (0x200 in SRAM)
        sts     ERR,r24
        ret                     ;back

;-------------------------------------------------------------------------------
; IPH   Monitor commands use binary values (shortened command!)
;-------------------------------------------------------------------------------
IPH:    ldi     r24,0x91        ;send 0x91 0x0D
        rcall   SNDCHR
        ldi     r24,0x0D
        rcall   SNDCHR
        rcall   RCVCHR          ;test monitor prompt ">" (0x3e 0x0d)
        cpi     r24,0x3E        ;if no prompt, set ERRorbyte (ERR)
        brne    IPHE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    IPHE
        ret                     ;back
IPHE:   lds     r24,ERR
        ori     r24,0x01        ;set Bit 0 in ERRorbyte
        sts     ERR,r24
        ret                     ;back

;-------------------------------------------------------------------------------
; SBD   Change monitor baud rate (Set Baud rate Divisor)(shortened command!)
;       Baud rate divisor in BRDIV (3 Bytes in SRAM)
;       TSTBRC is needed after change
;-------------------------------------------------------------------------------
SBD:    ldi     r24,0x14       ;send 0x14 0x20 divisor 0x0D
        rcall   SNDCHR
        ldi     r24,0x20
        rcall   SNDCHR
        lds     r24,BRDIV      ;Divisor in SRAM
        rcall   SNDCHR          ;1 Byte in BRDIV
        lds     r24,BRDIV+1    ;2 Byte in BRDIV+1
        rcall   SNDCHR          ;3 Byte in BRDIV+2
        lds     r24,BRDIV+2
        rcall   SNDCHR
        ldi     r24,0x0D
        rcall   SNDCHR
        rcall   RCVCHR          ;test monitor prompt ">" (0x3e 0x0d)
        cpi     r24,0x3E       ;if no prompt, set ERRorbyte (ERR)
        brne    SBDE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    SBDE
        ret                     ;back
SBDE:   lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte (0x200 in SRAM)
        sts     ERR,r24
        ret                     ;back

;-------------------------------------------------------------------------------
; TSTBRC   Test return prompt after baud rate change (No VDrive Command)
;-------------------------------------------------------------------------------
TSTBRC: ;ldi     r24,0x0D
        rcall   SNDCHR
        rcall   RCVCHR         ;test monitor prompt ">" (0x3e 0x0d)
        cpi     r24,0x3E       ;if no prompt, set ERRorbyte (ERR)
        brne    TSTBRE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    TSTBRE
        ret                     ;back
TSTBRE: lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte
        sts     ERR,r24
        ret                     ;back

;-------------------------------------------------------------------------------
; TSTIFD   Test if USB-Stick present (No VDrive Command)
; Returns 1 in r24 if Disk present (0 if no disk)
;-------------------------------------------------------------------------------
TSTIFD: ldi     r24,0x0D
        rcall   SNDCHR
        rcall   RCVCHR         ;test monitor prompt ">" (0x3e 0x0d)
        cpi     r24,0x3E       ;if no prompt, set ERRorbyte (ERR)
        brne    TSTIND
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    TSTIFE
        ldi     r24,1
        ret                     ;back
TSTIND: cpi     r24,0x4E        ;test if No Disk "ND" (0x4Ee 0x44 0x0d)
        brne    TSTIFE
        rcall   RCVCHR
        cpi     r24,0x44
        brne    TSTIFE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    TSTIFE
        ldi     r24,0
        ret                     ;back
TSTIFE: lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte
        sts     ERR,r24
        ret                     ;back



;-------------------------------------------------------------------------------
; FWV   Return FirmWare Version (shortened command!)
;       (12 Bytes in SRAM Table (first Byte in FWVER))
;-------------------------------------------------------------------------------
FWV:    ldi     r24,0x13       ;send 0x13 0x0D
        rcall   SNDCHR
        ldi     r24,0x0D
        rcall   SNDCHR
        rcall   RCVCHR          ;cr (0x0d)
        rcall   RCVCHR          ;M
        rcall   RCVCHR          ;A
        rcall   RCVCHR          ;I
        rcall   RCVCHR          ;N
        rcall   RCVCHR          ;space (0x20)
        rcall   RCVCHR          ;d
        sts     FWVER,r24
        rcall   RCVCHR          ;d
        sts     FWVER+1,r24
        rcall   RCVCHR          ;dot (0x2e)
        rcall   RCVCHR          ;d
        sts     FWVER+2,r24
        rcall   RCVCHR          ;d
        sts     FWVER+3,r24
        rcall   RCVCHR          ;A
        sts     FWVER+4,r24
        rcall   RCVCHR          ;A
        sts     FWVER+5,r24
        rcall   RCVCHR          ;A
        sts     FWVER+6,r24
        rcall   RCVCHR          ;A
        sts     FWVER+7,r24
        rcall   RCVCHR          ;A
        sts     FWVER+8,r24
        rcall   RCVCHR          ;cr (0x0d)
        rcall   RCVCHR          ;R
        rcall   RCVCHR          ;P
        rcall   RCVCHR          ;R
        rcall   RCVCHR          ;G
        rcall   RCVCHR          ;space (0x20)
        rcall   RCVCHR          ;d
        sts     FWVER+9,r24
        rcall   RCVCHR          ;dot (0x2e)
        rcall   RCVCHR          ;d
        sts     FWVER+10,r24
        rcall   RCVCHR          ;d
        sts     FWVER+11,r24
        rcall   RCVCHR          ;R
        rcall   RCVCHR          ;cr (0x0d)
        rcall   RCVCHR          ;test monitor prompt ">" (0x3e 0x0d)
        cpi     r24,0x3E       ;if no prompt, set ERRorbyte (ERR)
        brne    FWVE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    FWVE
        ret                     ;back
FWVE:   lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte (0x200 in SRAM)
        sts     ERR,r24
        ret                     ;back

;-------------------------------------------------------------------------------
;Echo E for synchronisation (shortened command!)
;-------------------------------------------------------------------------------
BIGE:   ldi     r24,0x45       ;send 0x45 0x0D
        rcall   SNDCHR
        ldi     r24,0x0D
        rcall   SNDCHR
        rcall   RCVCHR          ;test synchronisation ok "E" (0x45 0x0d)
        cpi     r24,0x45       ;if no "E", set ERRorbyte (ERR)
        brne    BIGEE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    BIGEE
        ret                     ;back
BIGEE:  lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte (0x200 in SRAM)
        sts     ERR,r24
        ret                     ;back
        ret

;-------------------------------------------------------------------------------
;Echo e for synchronisation (shortened command!)
;-------------------------------------------------------------------------------
LLEE:   ldi     r24,0x65       ;send 0x65 0x0D
        rcall   SNDCHR
        ldi     r24,0x0D
        rcall   SNDCHR
        rcall   RCVCHR          ;test synchronisation ok "e" (0x65 0x0d)
        cpi     r24,0x65       ;if no "e", set ERRorbyte (ERR)
        brne    LLEEE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    LLEEE
        ret                     ;back
LLEEE:  lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte (0x200 in SRAM)
        sts     ERR,r24
        ret                     ;back

;-------------------------------------------------------------------------------
;DIR    Listen Directory (shortened command!)
;       19 or more filenames are stored in SRAM (up to 255Byte)
;       first Byte (DIRLST+0) contains number of files
;-------------------------------------------------------------------------------
DIR:    push    XL              ;save pointer
        push    XH
        clr     r24             ;files are counted,and the number is stored in
        sts     DIRLST,r24      ;DIRLST; initialise DIRLST with zero
        ldi     XL,LOW(DIRLST+1) ;Filenames are stored in SRAM beginning with
        ldi     XH,HIGH(DIRLST+1) ;DIRLST+1, initialise pointer
        ldi     r24,0x01        ;send 0x01 0x0D (DIR)
        rcall   SNDCHR
        ldi     r24,0x0D
        rcall   SNDCHR
        rcall   RCVCHR          ;first character is 0x0D, will be ignored
DIR1:   rcall   RCVCHR          ;load character
        st      X+,r24          ;store character and increment pointer
        tst     XL              ;if 256 Byte stored (0-255) then stop it (DIRE2)
        breq    DIRE2
        cpi     r24,0x0D        ;if not filname end? go to DIR2
        brne    DIR2            ;else increment DIRLST and look for next char.
        lds     r24,DIRLST
        inc     r24
        sts     DIRLST,r24
        rjmp    DIR1
DIR2:   cpi     r24,0x3E        ;if no prompt, read next character
        brne    DIR1
        rcall   RCVCHR          ;if prompt test for carriage return
        cpi     r24,0x0D
        brne    DIRE
        rjmp    DIR3            ;back to main
DIRE:   lds     r24,ERR
        ori     r24,0x01        ;set Bit 0 in ERRorbyte (0x200 in SRAM)
        sts     ERR,r24
        rjmp    DIR3            ;back
DIRE2:  lds     r24,ERR         ;Bit 1 in Errorb. = "More than 20 files in dir"
        ori     r24,0x02        ;set Bit 1 in ERRorbyte
        sts     ERR,r24
DIR3:   pop     XH              ;restore pointer
        pop     XL
        ret                     ;back

;-------------------------------------------------------------------------------
;DIRF   List specified filesize (shortened command!)
;       filename in FNAME, filesize in FSIZE (SRAM)
;-------------------------------------------------------------------------------
DIRF:   push    XL
        push    XH
        ldi     XL,LOW(FNAME)   ;initialise pointer
        ldi     XH,HIGH(FNAME)  ;
        ldi     r24,0x01       ;send 0x01 0x20 (DIR filename)
        rcall   SNDCHR
        ldi     r24,0x20
        rcall   SNDCHR
DIRF1:  ld      r24,X+
        rcall   SNDCHR
        cpi     r24,0x0D
        brne    DIRF1
        rcall   RCVCHR          ;first r24acter is 0x0D, will be ignored
DIRF2:  rcall   RCVCHR          ;read filename and ignore it
        cpi     r24,0x20       ;filename ends with space (0x20)
        brne    DIRF2
        rcall   RCVCHR          ;read filesize and store it
        sts     FSIZE,r24
        rcall   RCVCHR
        sts     FSIZE+1,r24
        rcall   RCVCHR
        sts     FSIZE+2,r24
        rcall   RCVCHR
        sts     FSIZE+3,r24
        rcall   RCVCHR          ;last r24acter is 0x0D, will be ignored
        rcall   RCVCHR          ;test monitor prompt ">" (0x3e 0x0d)
        cpi     r24,0x3E       ;if no prompt, set ERRorbyte (ERR)
        brne    DIRFE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    DIRFE
        pop     XH              ;restore pointer
        pop     XL
        ret                     ;back
DIRFE:  lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte (0x200 in SRAM)
        sts     ERR,r24
        pop     XH              ;restore pointer
        pop     XL
        ret                     ;back




;-------------------------------------------------------------------------------
;OPW    Open a file for writing (shortened command!)
;       Filename (has to end with 0x0D!) in SRAM: FNAME
;-------------------------------------------------------------------------------
OPW:    push    XL
        push    XH
        ldi     XL,LOW(FNAME)   ;initialise pointer
        ldi     XH,HIGH(FNAME)  ;
        ldi     r24,0x09       ;send 0x09 0x20 (OPW)
        rcall   SNDCHR
        ldi     r24,0x20
        rcall   SNDCHR
OPW1:   ld      r24,X+
        rcall   SNDCHR
        cpi     r24,0x0D
        brne    OPW1
        rcall   RCVCHR          ;test monitor prompt ">" (0x3e 0x0d)
        cpi     r24,0x3E       ;if no prompt, set ERRorbyte (ERR)
        brne    OPWE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    OPWE
        pop     XH              ;restore pointer
        pop     XL
        ret                     ;back
OPWE:   lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte (0x200 in SRAM)
        sts     ERR,r24
        pop     XH              ;restore pointer
        pop     XL
        ret                     ;back

;-------------------------------------------------------------------------------
;WRF    Write bytes to opened file (shortened command!)
;       Number of bytes in SRAM: WRFNB
;-------------------------------------------------------------------------------
WRF:    push    XL
        push    XH
        ldi     XL,LOW(WDATA)   ;initialise pointer
        ldi     XH,HIGH(WDATA)  ;
        ldi     r24,0x08       ;send 0x08 0x20 (WRF)
        rcall   SNDCHR
        ldi     r24,0x20
        rcall   SNDCHR
        clr     r24
        rcall   SNDCHR
        rcall   SNDCHR
        rcall   SNDCHR
        lds     r24,WRFNB
        rcall   SNDCHR
        ldi     r24,0x0D
        rcall   SNDCHR
WRF1:   ld      r24,X+
        rcall   SNDCHR
        lds     r24,WRFNB      ;test if all Bytes send
        cp      XL,r24
        brne    WRF1
        rcall   RCVCHR          ;test monitor prompt ">" (0x3e 0x0d)
        cpi     r24,0x3E       ;if no prompt, set ERRorbyte (ERR)
        brne    WRFE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    WRFE
        pop     XH              ;restore pointer
        pop     XL
        ret                     ;back
WRFE:   lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte (0x200 in SRAM)
        sts     ERR,r24
        pop     XH              ;restore pointer
        pop     XL
        ret                     ;back

;-------------------------------------------------------------------------------
;CLF    Close a currently opened file (shortened command!)
;       Filename (has to end with 0x0D!) in SRAM: FNAME
;-------------------------------------------------------------------------------
CLF:    push    XL
        push    XH
        ldi     XL,LOW(FNAME)   ;initialise pointer
        ldi     XH,HIGH(FNAME)  ;
        ldi     r24,0x0A       ;send 0x0A 0x20 (CLF)
        rcall   SNDCHR
        ldi     r24,0x20
        rcall   SNDCHR
CLF1:   ld      r24,X+
        rcall   SNDCHR
        cpi     r24,0x0D
        brne    CLF1
        rcall   RCVCHR          ;test monitor prompt ">" (0x3e 0x0d)
        cpi     r24,0x3E       ;if no prompt, set ERRorbyte (ERR)
        brne    CLFE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    CLFE
        pop     XH              ;restore pointer
        pop     XL
        ret                     ;back
CLFE:   lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte (0x200 in SRAM)
        sts     ERR,r24
        pop     XH              ;restore pointer
        pop     XL
        ret                     ;back

;-------------------------------------------------------------------------------
;OPR    Opens a file for reading (shortened command!)
;       Filename (has to end with 0x0D!) in SRAM: FNAME
;-------------------------------------------------------------------------------
OPR:    push    XL
        push    XH
        ldi     XL,LOW(FNAME)   ;initialise pointer
        ldi     XH,HIGH(FNAME)  ;
        ldi     r24,0x0E       ;send 0x0E 0x20 (OPR)
        rcall   SNDCHR
        ldi     r24,0x20
        rcall   SNDCHR
OPR1:   ld      r24,X+
        rcall   SNDCHR
        cpi     r24,0x0D
        brne    OPR1
        rcall   RCVCHR          ;test monitor prompt ">" (0x3e 0x0d)
        cpi     r24,0x3E       ;if no prompt, set ERRorbyte (ERR)
        brne    OPRE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    OPRE
        pop     XH              ;restore pointer
        pop     XL
        ret                     ;back
OPRE:   lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte (0x200 in SRAM)
        sts     ERR,r24
        pop     XH              ;restore pointer
        pop     XL
        ret                     ;back

;-------------------------------------------------------------------------------
;RDF    Read bytes from opened file (shortened command!)
;       Number of bytes in SRAM: RDFNB
;-------------------------------------------------------------------------------
RDF:    push    XL
        push    XH
        ldi     XL,LOW(RDATA)   ;initialise pointer
        ldi     XH,HIGH(RDATA)  ;
        ldi     r24,0x0B       ;send 0x0B 0x20 (RDF)
        rcall   SNDCHR
        ldi     r24,0x20
        rcall   SNDCHR
        clr     r24
        rcall   SNDCHR
        rcall   SNDCHR
        rcall   SNDCHR
        lds     r24,RDFNB
        rcall   SNDCHR
        ldi     r24,0x0D
        rcall   SNDCHR
RDF1:   rcall   RCVCHR
        st      X+,r24
        lds     r24,RDFNB      ;test if all Bytes received
        cp      XL,r24
        brne    RDF1
        rcall   RCVCHR          ;test monitor prompt ">" (0x3e 0x0d)
        cpi     r24,0x3E       ;if no prompt, set ERRorbyte (ERR)
        brne    RDFE
        rcall   RCVCHR
        cpi     r24,0x0D
        brne    RDFE
        pop     XH              ;restore pointer
        pop     XL
        ret                     ;back
RDFE:   lds     r24,ERR
        ori     r24,0x01       ;set Bit 0 in ERRorbyte (0x200 in SRAM)
        sts     ERR,r24
        pop     XH              ;restore pointer
        pop     XL
        ret                     ;back

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Unterprogramme fuer die serielle Schnittstelle
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;-------------------------------------------------------------------------------
;Send a Character (r24)
;-------------------------------------------------------------------------------
SNDCHR: push    r25
SNDCHL: lds     r25,UCSR1A
        sbrs    r25,UDRE1       ;wait until UDREmpty set
        rjmp    SNDCHL          ;(if not write will be ignored)
        sts     UDR1,r24        ;r24 is parameter
        pop     r25
        ret                     ;back

;-------------------------------------------------------------------------------
;Receive a Character (r24)
;-------------------------------------------------------------------------------
RCVCHR: lds     r24,UCSR1A
        sbrs    r24,RXC1        ;wait until RXC (Receive Complete) set
        rjmp    RCVCHR          ;(polling)
        lds     r24,UCSR1A      ;read UCSR1A to check 3 ERRflags
        andi    r24,0b00011100  ;mask 3 ERRflags
        brne    RCVCHE          ;if ERR branch to RCVCHRE(rror)
        lds     r24,UDR1        ;read databyte
        ret                     ;back
RCVCHE: sts     ERR,r24         ;ERRorbyte (ERR in SRAM)
        ret                     ;back

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       ;Tabellen im Flash
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
HEADL:  .DB "DATUM",0x09,"ZEIT",0x09,"TEMPERATUR [°C]",0x09,"FEUCHTIGKEIT"
        .DB " [%]",0x09,"C02 [ppm]",0x0D,0x0D,0,0      ;55-Zeichen mit 2 CR
FNAMEF: .DB "  _  _  .txt",0x0D,0                    ;13-Zeichen mit CR